@@ -9,7 +9,7 @@
 
 A socket compatible interface with the Wiznet5k module.
 
-* Author(s): ladyada, Brent Rubell, Patrick Van Oosterwijck
+* Author(s): ladyada, Brent Rubell, Patrick Van Oosterwijck, Adam Cummick
 
 """
 import gc
@@ -108,11 +108,31 @@ class socket:
         if self._socknum == SOCKET_INVALID:
             raise RuntimeError("Failed to allocate socket.")
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self.disconnect()
+        stamp = time.monotonic()
+        while self.status == adafruit_wiznet5k.SNSR_SOCK_FIN_WAIT:
+            if time.monotonic() - stamp > 1000:
+                raise RuntimeError("Failed to disconnect socket")
+        self.close()
+        stamp = time.monotonic()
+        while self.status != adafruit_wiznet5k.SNSR_SOCK_CLOSED:
+            if time.monotonic() - stamp > 1000:
+                raise RuntimeError("Failed to close socket")
+
     @property
     def socknum(self):
         """Returns the socket object's socket number."""
         return self._socknum
 
+    @property
+    def status(self):
+        """Returns the status of the socket"""
+        return _the_interface.socket_status(self.socknum)[0]
+
     @property
     def connected(self):
         """Returns whether or not we are connected to the socket."""
@@ -147,10 +167,16 @@ class socket:
         return self._buffer
 
     def bind(self, address):
-        """Bind the socket to the listen port, we ignore the host.
-        :param tuple address: local socket as a (host, port) tuple, host is ignored.
+        """Bind the socket to the listen port, if host is specified the interface
+        will be reconfigured to that IP.
+        :param tuple address: local socket as a (host, port) tuple.
         """
-        _, self._listen_port = address
+        if address[0] is not None:
+            ip_address = _the_interface.unpretty_ip(address[0])
+            current_ip, subnet_mask, gw_addr, dns = _the_interface.ifconfig
+            if ip_address != current_ip:
+                _the_interface.ifconfig = (ip_address, subnet_mask, gw_addr, dns)
+        self._listen_port = address[1]
 
     def listen(self, backlog=None):
         """Listen on the port specified by bind.
@@ -160,6 +186,35 @@ class socket:
         _the_interface.socket_listen(self.socknum, self._listen_port)
         self._buffer = b""
 
+    def accept(self):
+        """Accept a connection. The socket must be bound to an address and listening for
+        connections. The return value is a pair (conn, address) where conn is a new
+        socket object usable to send and receive data on the connection, and address is
+        the address bound to the socket on the other end of the connection.
+        """
+        stamp = time.monotonic()
+        while self.status not in (
+            adafruit_wiznet5k.SNSR_SOCK_SYNRECV,
+            adafruit_wiznet5k.SNSR_SOCK_ESTABLISHED,
+        ):
+            if self._timeout > 0 and time.monotonic() - stamp > self._timeout:
+                return None
+            if self.status == adafruit_wiznet5k.SNSR_SOCK_CLOSED:
+                self.close()
+                self.listen()
+
+        new_listen_socknum, addr = _the_interface.socket_accept(self.socknum)
+        current_socknum = self.socknum
+        # Create a new socket object and swap socket nums so we can continue listening
+        client_sock = socket()
+        client_sock._socknum = current_socknum  # pylint: disable=protected-access
+        self._socknum = new_listen_socknum  # pylint: disable=protected-access
+        self.bind((None, self._listen_port))
+        self.listen()
+        while self.status != adafruit_wiznet5k.SNSR_SOCK_LISTEN:
+            raise RuntimeError("Failed to open new listening socket")
+        return client_sock, addr
+
     def connect(self, address, conntype=None):
         """Connect to a remote socket at address. (The format of address depends
         on the address family — see above.)
